import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.util.*;

/**
 * KMap.java 1.0 02/11/12
 * <br><br>
 * Author       : James McMahon
 * <br><br>
 * Answer Checker Class
 * <br><br>
 * <ul>
 *   <li> Provides the means to build up a minimal term (computer). </li>
 *   <li> To check the user's term is valid. (Does it represents the KMap?) </li>
 *   <li> To check the user's term is right. (minimal KMap) </li>
 * </ul>
 * <br>           
 * <br>		      
 * Created      : 13/11/2001<br>
 * Last Updated : 13/11/2001
 */

public class AnswerChecker {
	
	//The unicode modifer values for overbar
	private final String xBarValue = "x\u0304";
	private final String yBarValue = "y\u0304";
	private final String zBarValue = "z\u0304";
	
	//The lookup table which holds 26 KMaps and the terms associated with them
	//contains KMap then term
	private Object[][] lookUpTable = new Object[26][2]; 
	
	/**
	 * Creates the answer checker by populating the look up table
	 */
	public AnswerChecker() {
			
		//initilise the lookUpTable to hold the necessary values
		populateSingleTerms();
		populateDoubleTerms();
		populateTripleTerms();		
	}
	
	/** 
	 * Checks the users answer.	The answer to question could be correct,partially correct, or wrong.<br><br>
	 * 
	 * @param userTerms The term the user thinks is correct and wants to check
	 * @param questionKmap The actual KMap the term is supposed to represent
	 * 
	 * @return 0 indicates correct
	 * @return 1 indicates answer term represents the questionKMap but isn't minimal
	 * @return 2 indicates the answer is wrong
	 */
	public int checkAnswer(String userTerms, KMapType questionKMap) {

		String computerAnswer = buildComputerAnswer(questionKMap);
		
		if (userTerms.equals(computerAnswer)) {//then the terms match and don't need to do anymore work
			return 0;
		}
	    else {//answers don't match so build the user's kmap to see if it corresponds
			KMapType userKMap = buildUserKMap(userTerms);
		
			if (questionKMap.equals(userKMap)) {
				//code added because of the problem with the look up table in that
				//it doesn't always deal properly with KMaps with only double terms in the min term
				
				//we now know that the user term matches the KMap - so now we ned to ensure it is less
				//than the number of terms in the computerAnswer - if it is then the answer can be considered correct
				
				//this is a bit of a cheat and relies on the fact that answer term of the computer is going to be
				//considered as the best possible - even if it isn't
				//in this way if the user finds a better term, then we can assume that they are smarter than 
				//the look up table algorithm and take there answer as the best
				
				int numUserTerm = (parseTerms(userTerms)).size();//the number of minterms in the users minimal term
				int numCompTerms = (parseTerms(computerAnswer)).size();//the number of minterms in the computers minimal term
				
				if (numUserTerm <= numCompTerms) {//although they don't match the user term makes the KMap and is less than 
												//the computers so we'll assume it is the best minimum that can be found
					return 0;
				}
				else {
					return 1; //because the userterms do match the KMap but isn't minimal or less than the computer answer
				}			
			}
		}
		//if we've reached here then the answer is totally wrong
		return 2;
	}
	
	/** 
	 * userTerm is the minimal term in the form "x+y+xyz" for  example.
	 * <br>
	 * This will then return the KMap representation of the terms
	 * 
	 * @param userTerms The userTerm the user has built up
	 * @return KMapType The Kmap that those userTerms represent
	 */
	private KMapType buildUserKMap(String userTerms) {
		
		KMapType userKMap = new KMapType(0,0,0,0,0,0,0,0);//the KMap that is to be returned 
														  //starts off all zeros
		KMapType tempKMap = new KMapType(0,0,0,0,0,0,0,0);
		
		Vector termList = new Vector(1);
		termList = parseTerms(userTerms);//gets an array of terms with out the '+'s

		for(int i = 0; i <termList.size(); i++) {	

			tempKMap = lookUpStringValue((String) termList.elementAt(i));//returns the KMap associated with this string
			userKMap = userKMap.combine(tempKMap);//combines the two KMaps by taking the OR value of each component
		}	
		return userKMap;
	}
	
	/** 
	 * Returns the userterms without the + values
	 * 
	 * @param userTerm The user's minimal term they have built up
	 * @return Vector Contains the list of the pure terms in the userTerm 
	 */
	private Vector parseTerms(String userTerm) {
		Vector singleTerms = new Vector(1);
			
		//create a string tokeniser to remove all the +'s
		StringTokenizer st = new StringTokenizer(userTerm, "+");

		while (st.hasMoreTokens()) {
			String term = st.nextToken();
			singleTerms.addElement(term);
		}		
		return singleTerms;
	}

	/** 
	 * Returns the minimalTerm as far as the computer can tell!
	 * 
	 * @param questionKMap The question kmap you want the minimal term of
	 * @return String The minimal answer term for the question kmap
	 */
	public String buildComputerAnswer(KMapType questionKMap) {
		
		int kMapEntries = 8; //number of entries in the K-Map

		Vector fullList = new Vector();

		//Loop through all of the entries and build up the sub list's for each entry
		for(int i = 0; i < kMapEntries; i++) {
			Vector list = new Vector(1);//re-initlise each time around the loop
			
			//the list will contain integer values which can be used to access the lookupTable
			//which means we don't also need to store the corresponding terms as well
			
			list	= buildSubList(questionKMap, i);
			//Shouldn't need to sort the sub list because the look up table might take care of it for us
	
			fullList.addElement(list);//build up a list of the eight sublist's
		}

		Vector sortedFullList = new Vector(1);

		sortedFullList = sortSubLists(fullList);//Sorts the full list so that the result is a 
										   	    //list with all of the shortest sub list's first
										   	    //N.B. Leaving the sub list's themselves in the same order
		int counter = 0;	
		KMapType builtKMap = new KMapType(0,0,0,0,0,0,0,0);//blank KMap
		Vector indexList = new Vector(1); //stores the indexes to the lookup table
										  //to help stop duplicates	
											
		//next loop through each sub list in the sorted full list until we have a KMap 
		//that equals the question KMap
		while(!questionKMap.equals(builtKMap) && counter < kMapEntries) {
			
			Vector subList     = new Vector(1);
			KMapType partKMap  = new KMapType(0,0,0,0,0,0,0,0);//blank KMap
					
			//want to ignore any zero length sublist's
			if ( ((Vector) sortedFullList.elementAt(counter)).size() != 0){
				subList  = (Vector)sortedFullList.elementAt(counter);

				//Get the index for the look up table
				int index = ((Integer) subList.elementAt(0)).intValue();

				

				partKMap = (KMapType) lookUpTable[index][0];
			
				if (partKMap.fitsInto(questionKMap)) {
					indexList.addElement(new Integer(index));
					builtKMap.combine(partKMap);
				}
			}
			counter++;//increment counter			
		}
		return getAnswerString(filterList(indexList));
	}
	
	/** 
	 * Returns a sorted Vector with any duplicates in removed
	 * 
	 * @param indexList The list of indexs into the lookup table that have been found to match
	 * @return Vector A list without any duplicates
	 */
	private Vector filterList(Vector indexList) {
	
	//So few elements (only ever 8) so do a simple bubble sort		
	    for (int i=0; i<indexList.size()-1; i++) {
      		for (int j=indexList.size()-1; j>i; j--) {		
				int one = ((Integer) indexList.elementAt(j-1)).intValue();
				int two = ((Integer) indexList.elementAt(j)).intValue();
				if ((one >= two)) {
        			Object tmp = indexList.elementAt(j-1);
         			indexList.setElementAt(indexList.elementAt(j), j-1);
       				indexList.setElementAt(tmp, j);;
        			// if they are equal then remove one of them 
        			// because we only need just one!
        			if (one == two) {
        				indexList.removeElementAt(j);
        			}
        		}
			}
		}	
		return indexList;
	}

	/** 
	 * Returns the string representation of the lookuptable values stored in indexList
	 * 
	 * @param indexList The indexes into the lookup table that represent the locations of the KMap parts to be built up
	 * @return String The actual answer term that is built from the indexlists indexes into the lookup table
	 */
	private String getAnswerString(Vector indexList) {
		
		String answer = ""; 
		
		//iterate through all the values in the indexList
		Iterator itr = indexList.iterator();
		
		while (itr.hasNext()) {
			if (answer.equals("")) {//got the first term so don't want +
				answer = (String) lookUpTable[((Integer)itr.next()).intValue()][1];	
			} else { //not first term so add a + as well
				answer = answer+"+"+(String) lookUpTable[((Integer)itr.next()).intValue()][1];
			}
		}
		return answer;
	}
	
	/** 
	 * Returns a vector containing a list of integer values that correspond to all
	 * values in the look up table that cover 'location' in the questionKMap from the
	 * lookup table
	 * 
	 * @param questionKMap The question KMap that is actually wanted to match
	 * @param location The point that we are looking at (i.e. 0,1,2,3,4,5,6,7 of the KMap locations)
	 * @return Vector A list of indexes into the lookup table
	 */
	private Vector buildSubList(KMapType questionKMap, int location) {
		
		Vector list = new Vector(1);
			if((questionKMap.valueAt(location) == 1)) {
			//need to find values that match this from the lookup table
				
				for(int i=0; i<lookUpTable.length;i++) {
					if ((((KMapType) lookUpTable[i][0]).fitsInto(questionKMap)) && (((KMapType) lookUpTable[i][0]).valueAt(location) == 1) ) {
						list.addElement(new Integer(i));//add the index of the lookup table that matches the location
					}
				}
			}
		return list;//N.B. this list may be NULL
	}
	
	/** 
	 * Returns a sorted fullList based on the number of items in the sublist of each location.<br>
	 * Null sublists will be first.
	 * 
	 * @param fullList The list you want to be sorted
	 * @return Vector The sorted list of terms that match
	 */
	private Vector sortSubLists(Vector fullList) {
		
		//So few elements (only ever 8) so do a simple bubble sort		
	    for (int i=0; i<fullList.size()-1; i++) {
      		for (int j=fullList.size()-1; j>i; j--) {		
				//need to re-initilise every time around the loop
				Vector one = new Vector(1); 
				Vector two = new Vector(1);
				
				one = (Vector) fullList.elementAt(j-1);
				two = (Vector) fullList.elementAt(j);
				
				if (one.size() > two.size()) {
        			Object tmp = fullList.elementAt(j-1);
         			fullList.setElementAt(fullList.elementAt(j), j-1);
       				fullList.setElementAt(tmp, j);;
        		}
			}
		}	
		return fullList;
	}
	
	/** 
	 * Returns the KMapType from the lookup table that corresponds to term.<br>
	 * A blank K-Map will be returned if the term isn't found.
	 * 
	 * @param term The term that you want to find in the look up table
	 * @return KMapType The KMap that corresponds to that term
	 */
	private KMapType lookUpStringValue(String term) {
		//loop through the lookup table until the term matches
		//then return the corresponding KMap

		for(int i = 0;i<lookUpTable.length;i++) {

			if(((String) lookUpTable[i][1]).equals(term)) {
				return (KMapType) lookUpTable[i][0];//returns the KMap that matches the term
			}
		}
		//if we've reached here then the KMap term isn't found in the
		//look up table - should never happen in normal operation as only all 1's and all 0's aren't covered
		//hence just return a blank K-Map1111111111111111
		return new KMapType(0,0,0,0,0,0,0,0);
	}

	/** 
	 * Puts the single terms into the lookup table 
	 */	
	private void populateSingleTerms() {
		///////////////////////////////////////////
		lookUpTable[0][0] = new KMapType(1,1,1,1,
										 0,0,0,0);
		lookUpTable[0][1] = "x";
		///////////////////////////////////////////
		lookUpTable[1][0] = new KMapType(0,0,0,0,
										 1,1,1,1);
		lookUpTable[1][1] = xBarValue;
		///////////////////////////////////////////
		lookUpTable[2][0] = new KMapType(1,1,0,0,
										 1,1,0,0);
		lookUpTable[2][1] = "y";
		///////////////////////////////////////////
		lookUpTable[3][0] = new KMapType(0,0,1,1,
										 0,0,1,1);
		lookUpTable[3][1] = yBarValue;
		///////////////////////////////////////////
		lookUpTable[4][0] = new KMapType(1,0,0,1,
										 1,0,0,1);
		lookUpTable[4][1] = "z";
		///////////////////////////////////////////
		lookUpTable[5][0] = new KMapType(0,1,1,0,
										 0,1,1,0);
		lookUpTable[5][1] = zBarValue;
		///////////////////////////////////////////
	}
		
	/** 
	 * Puts the double terms into the lookup table 
	 */
	private void populateDoubleTerms() {
		///////////////////////////////////////////
		lookUpTable[6][0] = new KMapType(1,1,0,0,
										 0,0,0,0);
		lookUpTable[6][1] = "xy";
		///////////////////////////////////////////
		lookUpTable[7][0] = new KMapType(0,0,1,1,
										 0,0,0,0);
		lookUpTable[7][1] = "x"+yBarValue;
		///////////////////////////////////////////
		lookUpTable[8][0] = new KMapType(1,0,0,1,
										 0,0,0,0);
		lookUpTable[8][1] = "xz";
		///////////////////////////////////////////
		lookUpTable[9][0] = new KMapType(0,1,1,0,
										 0,0,0,0);
		lookUpTable[9][1] = "x"+zBarValue;
		///////////////////////////////////////////
		lookUpTable[10][0] = new KMapType(0,0,0,0,
										  1,1,0,0);
		lookUpTable[10][1] = xBarValue+"y";
		///////////////////////////////////////////
		lookUpTable[11][0] = new KMapType(0,0,0,0,
										  0,0,1,1);
		lookUpTable[11][1] = xBarValue+yBarValue;
		///////////////////////////////////////////
		lookUpTable[12][0] = new KMapType(0,0,0,0,
										  1,0,0,1);
		lookUpTable[12][1] = xBarValue+"z";
		///////////////////////////////////////////
		lookUpTable[13][0] = new KMapType(0,0,0,0,
										  0,1,1,0);
		lookUpTable[13][1] = xBarValue+zBarValue;
		///////////////////////////////////////////
		lookUpTable[14][0] = new KMapType(1,0,0,0,
										  1,0,0,0);
		lookUpTable[14][1] = "yz";
		///////////////////////////////////////////
		lookUpTable[15][0] = new KMapType(0,1,0,0,
										  0,1,0,0);
		lookUpTable[15][1] = "y"+zBarValue;
		///////////////////////////////////////////
		lookUpTable[16][0] = new KMapType(0,0,0,1,
										  0,0,0,1);
		lookUpTable[16][1] = yBarValue+"z";
		///////////////////////////////////////////
		lookUpTable[17][0] = new KMapType(0,0,1,0,
										  0,0,1,0);
		lookUpTable[17][1] = yBarValue+zBarValue;
		///////////////////////////////////////////
	}
	
	/** 
	 * Puts the triple terms into the lookup table 
	 */
	private void populateTripleTerms() {
		///////////////////////////////////////////
		lookUpTable[18][0] = new KMapType(1,0,0,0,
										  0,0,0,0);
		lookUpTable[18][1] = "xyz";
		///////////////////////////////////////////
		lookUpTable[19][0] = new KMapType(0,1,0,0,
										  0,0,0,0);
		lookUpTable[19][1] = "xy"+zBarValue;
		///////////////////////////////////////////
		lookUpTable[20][0] = new KMapType(0,0,0,1,
										  0,0,0,0);
		lookUpTable[20][1] = "x"+yBarValue+"z";
		///////////////////////////////////////////
		lookUpTable[21][0] = new KMapType(0,0,1,0,
										  0,0,0,0);
		lookUpTable[21][1] = "x"+yBarValue+zBarValue;
		///////////////////////////////////////////
		lookUpTable[22][0] = new KMapType(0,0,0,0,
										  1,0,0,0);
		lookUpTable[22][1] = xBarValue+"yz";
		///////////////////////////////////////////
		lookUpTable[23][0] = new KMapType(0,0,0,0,
										  0,1,0,0);
		lookUpTable[23][1] = xBarValue+"y"+zBarValue;
		///////////////////////////////////////////
		lookUpTable[24][0] = new KMapType(0,0,0,0,
										  0,0,0,1);
		lookUpTable[24][1] = xBarValue+yBarValue+"z";
		///////////////////////////////////////////
		lookUpTable[25][0] = new KMapType(0,0,0,0,
										  0,0,1,0);
		lookUpTable[25][1] = xBarValue+yBarValue+zBarValue;
		///////////////////////////////////////////
	}	
}