import java.awt.*;
import java.applet.*;
import javax.swing.*;

/**
 * KMap.java 1.0 02/11/12
 * <br><br>
 * Author       : James McMahon
 * <br><br>
 * Answer Panel Class
 * <br><br>
 * Provides the checkboxes for users to select in order to build up a minimal term
 * <br><br>
 * Created      : 13/11/2001<br>
 * Last Updated : 13/11/2001 
 */

public class AnswerPanel extends JPanel {
	
	/**
	 * The answer checker panel
	 */
	private AnswerChecker ansChecker = new AnswerChecker();
	
	//The unicode modifer values for overbar
	private final String xBarValue = "x\u0304";
	private final String yBarValue = "y\u0304";
	private final String zBarValue = "z\u0304";

	//The minimum x,y,z plus !x,!y,!z
	private JCheckBoxToolTipped x    = new JCheckBoxToolTipped("x");
	private JCheckBoxToolTipped xBar = new JCheckBoxToolTipped(xBarValue);
	private JCheckBoxToolTipped y    = new JCheckBoxToolTipped("y");
	private JCheckBoxToolTipped yBar = new JCheckBoxToolTipped(yBarValue);
	private JCheckBoxToolTipped z    = new JCheckBoxToolTipped("z");
	private JCheckBoxToolTipped zBar = new JCheckBoxToolTipped(zBarValue);
	
	//THe minimum x,y,z with two selected
	private JCheckBoxToolTipped xy       = new JCheckBoxToolTipped("xy");
	private JCheckBoxToolTipped xyBar    = new JCheckBoxToolTipped("x"+yBarValue);
	private JCheckBoxToolTipped xz       = new JCheckBoxToolTipped("xz");
	private JCheckBoxToolTipped xzBar    = new JCheckBoxToolTipped("x"+zBarValue);
	private JCheckBoxToolTipped xBary    = new JCheckBoxToolTipped(xBarValue+"y");
	private JCheckBoxToolTipped xBaryBar = new JCheckBoxToolTipped(xBarValue+yBarValue);	
	private JCheckBoxToolTipped xBarz    = new JCheckBoxToolTipped(xBarValue+"z");
	private JCheckBoxToolTipped xBarzBar = new JCheckBoxToolTipped(xBarValue+zBarValue);
	private JCheckBoxToolTipped yz       = new JCheckBoxToolTipped("yz");
	private JCheckBoxToolTipped yzBar    = new JCheckBoxToolTipped("y"+zBarValue);
	private JCheckBoxToolTipped yBarz    = new JCheckBoxToolTipped(yBarValue+"z");
	private JCheckBoxToolTipped yBarzBar = new JCheckBoxToolTipped(yBarValue+zBarValue);
		
	//The minimum x,y,z with three selected
	private JCheckBoxToolTipped xyz          = new JCheckBoxToolTipped("xyz");
	private JCheckBoxToolTipped xyzBar 	     = new JCheckBoxToolTipped("xy"+zBarValue);	
    private JCheckBoxToolTipped xyBarz	     = new JCheckBoxToolTipped("x"+yBarValue+"z");
	private JCheckBoxToolTipped xyBarzBar	 = new JCheckBoxToolTipped("x"+yBarValue+zBarValue);
	private JCheckBoxToolTipped xBaryz	     = new JCheckBoxToolTipped(xBarValue+"yz");
	private JCheckBoxToolTipped xBaryzBar	 = new JCheckBoxToolTipped(xBarValue+"y"+zBarValue);
	private JCheckBoxToolTipped xBaryBarz    = new JCheckBoxToolTipped(xBarValue+yBarValue+"z");
	private JCheckBoxToolTipped xBaryBarzBar = new JCheckBoxToolTipped(xBarValue+yBarValue+zBarValue);
	
	/** 
	 * instructions for the user
	 */
	private JLabel     instructions = new JLabel("   Select the terms to produce your answer then try submit!");
	
	/** 
	 * Area for the answer term to be built up - Non User Editable!
	 */
	private JTextField answerField = new JTextField(30);
	
	private JPanel singleTerms = new JPanel();
	private JPanel doubleTerms = new JPanel();
	private JPanel tripleTerms = new JPanel();
	
	private JPanel answerFeedbackPanel = new JPanel();
	private JPanel checkBoxItems	   = new JPanel();
	
	private AnswerPanelListener checkBoxListener = new AnswerPanelListener(this);
	
	/**
	 * Creates the answer panel with all of the checkboxes
	 */
	public AnswerPanel() {
	
		this.setLayout(new BorderLayout());
		this.setPreferredSize(new Dimension(100,100));
		this.setBorder(BorderFactory.createTitledBorder("Answer Terms"));		
	
		answerFeedbackPanel.setPreferredSize(new Dimension(75,50));
		answerFeedbackPanel.setBorder(BorderFactory.createTitledBorder("Your K-Map Minimalisation Term..."));
		
		
		//Add the answer field to the feedback panel and make it non-editable
		answerField.setEditable(false);
		
		//add a sensible tooltip
		setAnswerFieldToolTips();
		answerFeedbackPanel.add(answerField);
		
		setUpBoxListeners();
		
		//Add the Checkboxes to the respective panels
		addSingleTerms();
		addDoubleTerms();
		addTripleTerms();
		
		//Take those panels and add them to the main checkbox panel
		//using the BorderLayout
		checkBoxItems.setLayout(new GridLayout(3,1));
		
		checkBoxItems.add(singleTerms);
		checkBoxItems.add(doubleTerms);
		checkBoxItems.add(tripleTerms);
		checkBoxItems.setToolTipText("Hint displays the corresponding KMap for the term");
		
		//Add the instructions & score panel & check box items to this main JPanel
		this.add(instructions, BorderLayout.NORTH);
		this.add(checkBoxItems, BorderLayout.CENTER);
		this.add(answerFeedbackPanel, BorderLayout.SOUTH);
	}
	
	/** 
	 * Adds the necessary listeners to all of the JCheckBoxToolTippeds
	 */
	public void setUpBoxListeners() {
	
		x.addItemListener(checkBoxListener);
		xBar.addItemListener(checkBoxListener);
		y.addItemListener(checkBoxListener);
		yBar.addItemListener(checkBoxListener);
		z.addItemListener(checkBoxListener);
		zBar.addItemListener(checkBoxListener);
	
		xy.addItemListener(checkBoxListener);
		xyBar.addItemListener(checkBoxListener);
		xz.addItemListener(checkBoxListener);
		xzBar.addItemListener(checkBoxListener);
		xBary.addItemListener(checkBoxListener);
		xBaryBar.addItemListener(checkBoxListener);
		xBarz.addItemListener(checkBoxListener);
		xBarzBar.addItemListener(checkBoxListener);
		yz.addItemListener(checkBoxListener);
		yzBar.addItemListener(checkBoxListener);
		yBarz.addItemListener(checkBoxListener);
		yBarzBar.addItemListener(checkBoxListener);
		
		xyz.addItemListener(checkBoxListener);
		xyzBar.addItemListener(checkBoxListener);
		xyBarz.addItemListener(checkBoxListener);
		xyBarzBar.addItemListener(checkBoxListener);
		xBaryz.addItemListener(checkBoxListener);
		xBaryzBar.addItemListener(checkBoxListener);
		xBaryBarz.addItemListener(checkBoxListener);
		xBaryBarzBar.addItemListener(checkBoxListener);
	}
	
	/** 
	 * Add the single terms to the single Jpanel
	 */
	private void addSingleTerms() {
		
		singleTerms.add(x);
		singleTerms.add(xBar); 
		singleTerms.add(y);    
		singleTerms.add(yBar); 
		singleTerms.add(z);    
		singleTerms.add(zBar); 	
	}
	
	/** 
	 * Add the double terms to the double JPanel
	 */
	private void addDoubleTerms() {
	
		doubleTerms.add(xy);       
		doubleTerms.add(xyBar);    
		doubleTerms.add(xz);  
		doubleTerms.add(xzBar);  
		doubleTerms.add(xBary);
		doubleTerms.add(xBaryBar); 	
		doubleTerms.add(xBarz);
		doubleTerms.add(xBarzBar); 
		doubleTerms.add(yz);  
		doubleTerms.add(yzBar);  
		doubleTerms.add(yBarz);
		doubleTerms.add(yBarzBar);
	}
	
	/** 
	 * Add the triple terms to the right JPanel
	 */	
	private void addTripleTerms() {
		
		tripleTerms.add(xyz);   
		tripleTerms.add(xyzBar); 	
		tripleTerms.add(xyBarz);
		tripleTerms.add(xyBarzBar);
		tripleTerms.add(xBaryz);
		tripleTerms.add(xBaryzBar);
		tripleTerms.add(xBaryBarz);
		tripleTerms.add(xBaryBarzBar);	
	}
	
	/** 
	 * Resets the checkboxes
	 */
	private void resetCheckBoxes() {
	
		x.setSelected(false);
		xBar.setSelected(false);
		y.setSelected(false);
		yBar.setSelected(false);
		z.setSelected(false);
		zBar.setSelected(false);
	
		xy.setSelected(false);
		xyBar.setSelected(false);
		xz.setSelected(false);
		xzBar.setSelected(false);
		xBary.setSelected(false);
		xBaryBar.setSelected(false);
		xBarz.setSelected(false);
		xBarzBar.setSelected(false);
		yz.setSelected(false);
		yzBar.setSelected(false);
		yBarz.setSelected(false);
		yBarzBar.setSelected(false);
		
		xyz.setSelected(false);
		xyzBar.setSelected(false);
		xyBarz.setSelected(false);
		xyBarzBar.setSelected(false);
		xBaryz.setSelected(false);
		xBaryzBar.setSelected(false);
		xBaryBarz.setSelected(false);
		xBaryBarzBar.setSelected(false);
	}
	
	/** 
	 * Resets the answer term
	 */
	private void resetAnswerTerm() {
		answerField.setText("");
		
	}
	
	/** 
	 * Resets the entire answer panel, checkboxes and answerpanel
	 */
	public void resetAnswerPanel() {
		resetCheckBoxes();
		resetAnswerTerm();
	}
	
	/** 
	 * Generate and displays the answer term based on the currently selected checkboxes
	 */
	public void generateAnswerTerm() {
		
		//first clear the current anwer term
		resetAnswerTerm();
		
		//go through each checkbox and see if it is selected
		//if it is then build up the term...
		
		//single terms
		if (x.isSelected())    { addToAnswerFieldText(x.getText());}
		if (xBar.isSelected()) { addToAnswerFieldText(xBar.getText());}
		if (y.isSelected())    { addToAnswerFieldText(y.getText());}
		if (yBar.isSelected()) { addToAnswerFieldText(yBar.getText());}
		if (z.isSelected())    { addToAnswerFieldText(z.getText());}
	    if (zBar.isSelected()) { addToAnswerFieldText(zBar.getText());}
	    
	    //double terms
		if (xy.isSelected())       { addToAnswerFieldText(xy.getText());}	
		if (xyBar.isSelected())    { addToAnswerFieldText(xyBar.getText());}
		if (xz.isSelected())       { addToAnswerFieldText(xz.getText());}
		if (xzBar.isSelected())    { addToAnswerFieldText(xzBar.getText());}
		if (xBary.isSelected())    { addToAnswerFieldText(xBary.getText());}
		if (xBaryBar.isSelected()) { addToAnswerFieldText(xBaryBar.getText());}
		if (xBarz.isSelected())    { addToAnswerFieldText(xBarz.getText());}
		if (xBarzBar.isSelected()) { addToAnswerFieldText(xBarzBar.getText());}
		if (yz.isSelected()) 	   { addToAnswerFieldText(yz.getText());}
		if (yzBar.isSelected())    { addToAnswerFieldText(yzBar.getText());}
		if (yBarz.isSelected())    { addToAnswerFieldText(yBarz.getText());}
		if (yBarzBar.isSelected()) { addToAnswerFieldText(yBarzBar.getText()); }
		
		//triple terms
		if (xyz.isSelected())          { addToAnswerFieldText(xyz.getText());}
		if (xyzBar.isSelected())       { addToAnswerFieldText(xyzBar.getText());}
		if (xyBarz.isSelected())       { addToAnswerFieldText(xyBarz.getText());}
		if (xyBarzBar.isSelected())    { addToAnswerFieldText(xyBarzBar.getText());}
		if (xBaryz.isSelected())       { addToAnswerFieldText(xBaryz.getText());} 
		if (xBaryzBar.isSelected())    { addToAnswerFieldText(xBaryzBar.getText());}
		if (xBaryBarz.isSelected())    { addToAnswerFieldText(xBaryBarz.getText());}
		if (xBaryBarzBar.isSelected()) { addToAnswerFieldText(xBaryBarzBar.getText());}	
	}
	
	/** 
	 * Adds term to the end of the answer string
	 * 
	 * @param term The term to be added with or without a preceding +
	 */
	private void addToAnswerFieldText(String term) { 
	
		String temp = answerField.getText();
		if (!temp.equals("")) { //got a term so add to it
			temp = temp+"+"+term;
		} else { //not got a term hence this is the first 
				 //one and don't want a preceding +
			temp = term;
		}
		answerField.setText(temp);
		//as we've reset the text we now need to reset the tooltips
		setAnswerFieldToolTips();
	}
	
	/** 
	 * Sets a sensible tool tip based on contents of the minimal term string field
	 */
	private void setAnswerFieldToolTips() {
				
		if (answerField.getText().equals("")) { //don't have text so put in something else
			answerField.setToolTipText("Your term will build up here");
		} else {
			answerField.setToolTipText(answerField.getText());
		}	
	}
	
	/** 
	 * Removes all of the tooltips of the JCheckBoxesToolTipped. <br>
	 * N.B. setEnable(false) is not used because it would turn off all
	 *     tooltips and that isn't what we want. This way only resets the checkboxes
	 * <br><br>
	 * Used when user turns off hints
	 */
	public void removeToolTips() {
		
		String nullString = null;
		
		x.setToolTipText(nullString);
		xBar.setToolTipText(nullString);
		y.setToolTipText(nullString);
		yBar.setToolTipText(nullString);
		z.setToolTipText(nullString);
		zBar.setToolTipText(nullString);
	
		xy.setToolTipText(nullString);
		xyBar.setToolTipText(nullString);
		xz.setToolTipText(nullString);
		xzBar.setToolTipText(nullString);
		xBary.setToolTipText(nullString);
		xBaryBar.setToolTipText(nullString);
		xBarz.setToolTipText(nullString);
		xBarzBar.setToolTipText(nullString);
		yz.setToolTipText(nullString);
		yzBar.setToolTipText(nullString);
		yBarz.setToolTipText(nullString);
		yBarzBar.setToolTipText(nullString);
		
		xyz.setToolTipText(nullString);
		xyzBar.setToolTipText(nullString);
		xyBarz.setToolTipText(nullString);
		xyBarzBar.setToolTipText(nullString);
		xBaryz.setToolTipText(nullString);
		xBaryzBar.setToolTipText(nullString);
		xBaryBarz.setToolTipText(nullString);
		xBaryBarzBar.setToolTipText(nullString);
	}
	
	/** 
	 * Adds all of the tooltips to the JCheckBoxesToolTipped
	 * <br>
	 * Should be called when the user asks for hints
	 */
	public void addToolTips() {
		
		x.setToolTipText("1111\n0000");
		xBar.setToolTipText("0000\n1111");
		y.setToolTipText("1100\n1100");
		yBar.setToolTipText("0011\n0011");
		z.setToolTipText("1001\n1001");
		zBar.setToolTipText("0110\n0110");
	
		xy.setToolTipText("1100\n0000");
		xyBar.setToolTipText("0011\n0000");
		xz.setToolTipText("1001\n0000");
		xzBar.setToolTipText("0110\n0000");
		xBary.setToolTipText("0000\n1100");
		xBaryBar.setToolTipText("0000\n0011");
		xBarz.setToolTipText("0000\n1001");
		xBarzBar.setToolTipText("0000\n0110");
		yz.setToolTipText("1000\n1000");
		yzBar.setToolTipText("0100\n0100");
		yBarz.setToolTipText("0001\n0001");
		yBarzBar.setToolTipText("0010\n0010");
		
		xyz.setToolTipText("1000\n0000");
		xyzBar.setToolTipText("0100\n0000");
		xyBarz.setToolTipText("0001\n0000");
		xyBarzBar.setToolTipText("0010\n0000");
		xBaryz.setToolTipText("0000\n1000");
		xBaryzBar.setToolTipText("0000\n0100");
		xBaryBarz.setToolTipText("0000\n0001");
		xBaryBarzBar.setToolTipText("0000\n0010");
	}
	
	/** 
	 * Checks the answer to see if it is correct or not. The answer to question could be correct,partially correct, or wrong.<br>
	 * Answer term is taken automatically from the answerField of this panel to check against.
	 * <br>
	 * Used with the submit button.
	 *
	 * @return 0 indicates correct
	 * @return 1 indicates answer term represents the questionKMap but isn't minimal
	 * @return 2 indicates the answer is plain wrong
	 */
	public int checkAnswer(KMapType questionKMap) {
		return ansChecker.checkAnswer(answerField.getText(),questionKMap);
	}
	
	/** 
	 * Returns the answer that is built using the computer algorithm.
	 * <br>
	 * For use with the reveal button.
	 * 
	 * @param questionKMap The kmap you want the minimal term of
	 * @return String The minimal term as far as the computer algorithm is concerned
	 */
	public String getCorrectAnswer(KMapType questionKMap) {
		//resetAnswerPanel();
		return ansChecker.buildComputerAnswer(questionKMap);	
	}
}