import java.awt.*;
import java.applet.*;
import javax.swing.*;

/*
 * KMap.java 1.0 02/11/12
 * <br><br>
 * Author       : James McMahon
 * <br><br>
 * K-Map Applet Class
 * <br><br>
 * Created      : 12/11/2001
 * <br>
 * Last Updated : 12/11/2001
 */

public class KMap extends JApplet {
	
	/**
	 * The answer panel for the applet
	 */
	private AnswerPanel   ansPanel         = new AnswerPanel();
	
	/**
	 * The menuPanel of the applet
	 */
	private MenuPanel     menuPanel        = new MenuPanel(this);

	/**
	 * The question diaplay area of the applet
	 */
	private QuestionPanel questionPanel    = new QuestionPanel();

 	/**
 	 * The score display panel
 	 */
	private ScoreKeeper   scorePanel       = new ScoreKeeper();
	
	/**
	 * Not really a hint anymore, more of a tip. <br>
	 * Indicates to the user that the algorithm may not generate a unique minimal term
	 */
	private HintPanel	  hintPanel	       = new HintPanel();
	
	/**
	 * Allows a better layout
	 */
	private JPanel 		  tempEastHolder   = new JPanel(new BorderLayout());
	
	/**
	 * Allows a neater more compact layou on screen
	 */
	private JPanel		  tempCenterHolder = new JPanel(new BorderLayout());

	/**
	 *	The content pane of the JApplet
	 */
	private Container content = getContentPane();
	
	/** 
	 * boolean variables that help to keep track of what the user has been up to
	 * <br>
	 * Stores whether the user has asked for the answer to be revealed
	 */
	private boolean revealedAnswer	       = false;
	
	/**
	 * boolean variables that help to keep track of what the user has been up to
	 * <br>
	 * Stores whether the user has already submitted a correct answer
	 */
	private boolean submittedCorrectAnswer = false;

	/*
	 * boolean variables that help to keep track of what the user has been up to
	 * <br>
	 * Stores whether the users tooltips are on or not
	 */
	private boolean toolTipsOn		       = false;
	
	/**
	 * Initilises the applet with a random KMap as a question on screen
	 */
	public void init() {
		
		super.init();
		
		content.setLayout(new BorderLayout());
		
		//add the score holder and question panel to the temp panel
		//in order to make a nicer layout on screen
		tempEastHolder.add(questionPanel, BorderLayout.CENTER);
		tempEastHolder.add(scorePanel, BorderLayout.SOUTH);
		
		//add the answerpanel and the hintpane to the tempCenterPanel
		//in order to get a nicer looking layout
		tempCenterHolder.add(ansPanel, BorderLayout.CENTER);
		tempCenterHolder.add(hintPanel, BorderLayout.SOUTH);
		
		//Add the various panels to the applet's content pane
		content.add(tempCenterHolder, BorderLayout.CENTER);
		
		content.add(menuPanel, BorderLayout.SOUTH);
		content.add(tempEastHolder, BorderLayout.EAST);
	}

	/**
	 * No explicit threads are created so nothing special needs to be done in here
	 */
	public void start()	{
		super.start();
	}
	
	/**
	 * No explicit threads are created so nothing special needs to be done in here
	 */
	public void stop()	{
		super.stop();
	}
	
	/** 
	 * Puts a new KMap on the screen
	 */
	public void generateNewQuestion() {
		//reset the tracker variable
	 	revealedAnswer		   = false;
		submittedCorrectAnswer = false;
	
		questionPanel.generateQuestion();//will also show it on screen
		resetAllScreens();
		scorePanel.addQuestionAsked();
	}
	
	/** 
	 * Show the user the hint panel
	 */
	public void showHint() {
		if (toolTipsOn) {//turn them off
			JOptionPane.showMessageDialog(this,
			 			  "Tooltips for the checkboxes are now off",
			    	      "ToolTips OFF",
			        	  JOptionPane.INFORMATION_MESSAGE);
			toolTipsOn = false;
			ansPanel.removeToolTips();
		} else {//turn them on
			JOptionPane.showMessageDialog(this,
			 			  "Tooltips for the checkboxes are now on",
			    	      "ToolTips ON",
			        	  JOptionPane.INFORMATION_MESSAGE);
			toolTipsOn = true;
			ansPanel.addToolTips();
		}
	}
	
	/**  
	 * Show the user the answer to the current question
	 */
	public void showAnswer() {
		
		String answer = ansPanel.getCorrectAnswer(questionPanel.getCurrentKMapQuestion());
		revealedAnswer = true;

		JOptionPane.showMessageDialog(this,
					  "The answer is "+answer,
			          "The answer is ...",
			          JOptionPane.INFORMATION_MESSAGE);
	}
	
	/** 
	 * Checks the user's answer
	 */
	public void checkAnswer() {
		
		if (!revealedAnswer) {
		  	if (!submittedCorrectAnswer) {		
				int	result = ansPanel.checkAnswer(questionPanel.getCurrentKMapQuestion());

				if (result == 0) {//answer was right
					submittedCorrectAnswer = true;
					scorePanel.addQuestionRight();//update score tracker
					JOptionPane.showMessageDialog(this,
									  "Why don't you try another question",
							          "Well Done: Correct Answer",
							          JOptionPane.INFORMATION_MESSAGE);
				}
				else if (result == 1) {//answer was partially right
									   //the term fits the KMap but isn't minimal
					JOptionPane.showMessageDialog(this,
									  "Although your term matches the KMap it doesn't match the computers minimal term",
							          "Possibly incorrect answer, try to reduce your terms",
							          JOptionPane.INFORMATION_MESSAGE);
				}	
				else { //result must equal two so the answer is completely wrong
					JOptionPane.showMessageDialog(this,
									  "Sorry your answer term is wrong",
							          "Sorry : Incorrect answer, Please try again",
							          JOptionPane.ERROR_MESSAGE);
				}
			} 
			else {//user has already submitted this answer and got it right so stop him re-submitting
				JOptionPane.showMessageDialog(this,
						  "You've already submitted this answer!",
					      "Thinking of cheating?",
					      JOptionPane.INFORMATION_MESSAGE);		
			} 
		} 
		else {//user was given the answer so tell him to stop cheating
			JOptionPane.showMessageDialog(this,
					  "Don't you trust the answer that I gave you?",
			          "Thinking of cheating?",
			          JOptionPane.INFORMATION_MESSAGE);	
		}
	}
	
	/** 
	 * Resets the checkboxes and the user's answer term
	 */
	public void resetAllScreens() {
		ansPanel.resetAnswerPanel();	
	}
}
