import java.awt.*;
import java.applet.*;
import javax.swing.*;

/**
 * KMap.java 1.0 02/11/01
 * <br><br>
 * Author       : James McMahon
 * <br><br>
 * KMap Applet Class
 * <br><br>
 * The Menu Panel for the applet menu
 * <br><br>
 * Created      : 13/11/2001
 * <br>
 * Last Updated : 13/11/2001
 * <br>
 */

public class MenuPanel extends JPanel {
		
		/**
		 *	The main class used to allow access to the necessary functions.<br> 
		 *  This is used only to pass through to the button listener
		 */
		private KMap parent;
	    
	    /**
	     * Get a new question button
	     */
		private JButton newQuestion = new JButton("New");
		
	 	/**
	 	 * Display a help page
	 	 */	
		private JButton help        = new JButton("Help");
		
		/**
		 * Show a hint
		 */
		private JButton hint		= new JButton("Hint");
		
		/**
		 * Reveal the answer and pass the question by
		 */
		private JButton reveal      = new JButton("Reveal");
	
		/**
		 * Check the user's answer is correct
		 */
		private JButton check	    = new JButton("Submit");
	
		/**
		 * Reset the screen
		 */
		private JButton reset       = new JButton("Reset");
	
		/**
		 * Allows the screen to be refreshed if necessary
		 */
		private JButton refresh     = new JButton("Refresh");
	
		/**
		 *
		 */
		private MenuPanelListener mListener;
	
	    /**
	     * Creates the menupanel usually displayed at the bottom of the screen
	     * 
	     * @param parentOwner The main class that is passed into the listener for the buttons
	     */
		public MenuPanel(KMap parentOwner) {
		
			parent = parentOwner;
		
			this.setLayout(new FlowLayout());
			this.setBorder(BorderFactory.createEtchedBorder());
			
			addListeners(); //Add the MenuListener to all of the buttons
		
			//Add the tooltips to the buttons
			newQuestion.setToolTipText("Get's a new question");
			help.setToolTipText("Displays a help page");
			hint.setToolTipText("Get a hint on the answer");
			reveal.setToolTipText("Displays the correct answer");
			check.setToolTipText("Check if your answer is correct");
			reset.setToolTipText("Reset CheckBoxes to unselected");
			refresh.setToolTipText("Refresh the display");
			
			newQuestion.setMnemonic('n');
			help.setMnemonic('h');
			hint.setMnemonic('i');
			reveal.setMnemonic('r');
			check.setMnemonic('s');
			reset.setMnemonic('t');
			refresh.setMnemonic('f');
			
			// Add the buttons to the panel
			this.add(newQuestion);
			this.add(help);
			this.add(hint);
			this.add(reveal);
			this.add(check);
			this.add(reset);
			this.add(refresh);
		}	
		
		/**
		 * Adds a menuListener to each of the buttons
		 */
		public void addListeners() {
		
			mListener = new MenuPanelListener(parent);
		
			newQuestion.addActionListener(mListener); 
			help.addActionListener(mListener);     
			hint.addActionListener(mListener);
			reveal.addActionListener(mListener);
			check.addActionListener(mListener);
			reset.addActionListener(mListener);
			refresh.addActionListener(mListener);	
		}
}