import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * KMap.java 1.0 02/11/12
 * <br><br>
 * Author       : James McMahon
 * <br><br>
 * KMap Applet Class
 * <br><br>
 * The MenuListener to deal with all of the necessary actions 
 * <br><br>
 * N.B. Currently tightly coupled to the text of the JButtons in the MenuPanel
 * - change one then remember to change the other
 * <br><br>
 * Created      : 13/11/2001
 * <br>
 * Last Updated : 11/11/2001
 */
 
public class MenuPanelListener implements ActionListener {
	
	/**
	 * The parent KMap to allow access to the necessary functions which do all the work
	 */
	private KMap parent;
	
	/**
	 * Creates a listener that calls the functions of parentOwner (the main class) 
	 * when the buttons are clicked
	 * 
	 * @param parentOwner The main class to allow access to the relevant functions
	 */
	public MenuPanelListener(KMap parentOwner) {
		parent = parentOwner;
	}
	
	/**
	 * Deals with the user clicking the buttons
	 * 
	 * @param ActionEvent The event from a button that triggered the listener
	 */
	public void actionPerformed(ActionEvent ae) {

		if (ae.getActionCommand().equals("New")) {
			parent.generateNewQuestion();
		}
		else if (ae.getActionCommand().equals("Help")) { 
			new HelpPanel(parent);
		}	
		else if (ae.getActionCommand().equals("Hint")) {
			parent.showHint();
		}
		else if (ae.getActionCommand().equals("Reveal")) { 
			parent.showAnswer();
		}
		else if (ae.getActionCommand().equals("Submit")) {
			parent.checkAnswer();
		}
		else if (ae.getActionCommand().equals("Reset")) { 
			parent.resetAllScreens();
		}
		else if (ae.getActionCommand().equals("Refresh")) { 
			parent.repaint();
		}
	}
}