import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.util.*;


/**
 * KMap.java 1.0 02/11/01
 * <br>
 * Author       : James McMahon
 * <br><br>
 * KMap Applet Class
 * <br>
 * The Question Panel which displays the KMap
 * <br>
 * Also allows generation of a random K-Map for display
 * <br><br>
 * Being able to display a K-Map passd in from outside is due to the
 * need to be able to re-use this code in the Truth Table to Circuit Diagram applet
 * <br>
 * Created      : 14/11/2001
 * <br>
 * Last Updated : 14/11/2001
 */

public class QuestionPanel extends JPanel {
	
	/**
	 * The KMap that is the current question
	 */
	private KMapType question; 
	
	//Top row fields
	private JTextField top1 = new JTextField(2);
	private JTextField top2 = new JTextField(2);
	private JTextField top3 = new JTextField(2);
	private JTextField top4 = new JTextField(2);
	
	//bottom row fields
	private JTextField bottom1 = new JTextField(2);
	private JTextField bottom2 = new JTextField(2);
	private JTextField bottom3 = new JTextField(2);
	private JTextField bottom4 = new JTextField(2);
	
	//JLabels for the sides
	private JLabel x     = new JLabel("x");
	private JLabel xBar  = new JLabel("x\u0304");
	
	private JLabel y1    = new JLabel("y");
	private JLabel y2    = new JLabel("y");
	private JLabel y1Bar = new JLabel("y\u0304");
	private JLabel y2Bar = new JLabel("y\u0304");
	
	private JLabel z1    = new JLabel("z");
	private JLabel z2    = new JLabel("z");	
	private JLabel z1Bar = new JLabel("z\u0304");
	private JLabel z2Bar = new JLabel("z\u0304");
	
	private GridBagLayout gridBag;
	private GridBagConstraints cons;	
	
	/** 
	 * Generates the necessary infra structure for the K-Map
	 * and displpays a random one as the first question
	 */
	public QuestionPanel() {

		setUpScreen();
		
		generateQuestion();//generate and display new question

	}
	
	/** 
	 * Displays the input KMap as the current question
	 * 
	 * @param question The kmap that you want to ask the user to simplify
	 */
	public QuestionPanel(KMapType question) {
		this.question = question;
		setUpScreen();
		displayKMap(question);
	}
	
	/** 
	 * Sets up the screen data
	 */
	public void setUpScreen(){
				
		this.setPreferredSize(new Dimension(150,150));	
		this.setBorder(BorderFactory.createTitledBorder("Simplify this K-Map"));		
		
		//set up the grid bag layou
		gridBag = new GridBagLayout();
		setLayout(gridBag);
		
		setUpLabels();
		setUpTextFields();
	}
	
	/** 
	 * Sets up the labels that are needed for the display
	 */
	private void setUpLabels() {
		
		cons = getDefaultConstraints();
		
		//Set up the column labels
		cons.gridx = 0;
	    cons.gridy = 1;
		gridBag.setConstraints(x, cons);
		add(x);
		
		cons.gridx = 0;
	    cons.gridy = 2;
		gridBag.setConstraints(xBar, cons);
		add(xBar);
		
		//Set up the top row labels
		cons.gridx = 1;
	    cons.gridy = 0;
		gridBag.setConstraints(y1, cons);
		add(y1);
		
		cons.gridx = 2;
	    cons.gridy = 0;
		gridBag.setConstraints(y2, cons);
		add(y2);
		
		cons.gridx = 3;
	    cons.gridy = 0;
		gridBag.setConstraints(y1Bar, cons);
		add(y1Bar);
		
		cons.gridx = 4;
	    cons.gridy = 0;
		gridBag.setConstraints(y2Bar, cons);
		add(y2Bar);
		
		//Set up the bottom row labels
		cons.gridx = 1;
	    cons.gridy = 3;
		gridBag.setConstraints(z1, cons);
		add(z1);
		
		cons.gridx = 2;
	    cons.gridy = 3;
		gridBag.setConstraints(z2Bar, cons);
		add(z2Bar);
		
		cons.gridx = 3;
	    cons.gridy = 3;
		gridBag.setConstraints(z1Bar, cons);
		add(z1Bar);
		
		cons.gridx = 4;
	    cons.gridy = 3;
		gridBag.setConstraints(z2, cons);
		add(z2);
	}
	
	/** 
	 * Sets up the textfields that are needed for the display
	 */
	private void setUpTextFields() {
		
		cons = getDefaultConstraints();
		
		//set the question boxes non editable and make them look white ...
		top1.setEditable(false);
		top2.setEditable(false);
		top3.setEditable(false);
		top4.setEditable(false);
		
		top1.setBackground(Color.white);
		top2.setBackground(Color.white);
		top3.setBackground(Color.white);
		top4.setBackground(Color.white);
		
		bottom1.setEditable(false);
		bottom2.setEditable(false);
		bottom3.setEditable(false);
		bottom4.setEditable(false);
		
		bottom1.setBackground(Color.white);
		bottom2.setBackground(Color.white);
		bottom3.setBackground(Color.white);
		bottom4.setBackground(Color.white);
	
		//Set up the first row
		cons.gridx = 1;
	    cons.gridy = 1;
	    cons.insets     = new Insets(5, 0, 0, 0);
		gridBag.setConstraints(top1, cons);
		add(top1);
		
		cons.gridx = 2;
	    cons.gridy = 1;
	    cons.insets     = new Insets(5, 0, 0, 0);
		gridBag.setConstraints(top2, cons);
		add(top2);
		
		cons.gridx = 3;
	    cons.gridy = 1;
	    cons.insets     = new Insets(5, 0, 0, 0);
		gridBag.setConstraints(top3, cons);
		add(top3);
		
		cons.gridx = 4;
	    cons.gridy = 1;
	    cons.insets     = new Insets(5, 0, 0, 0);
		gridBag.setConstraints(top4, cons);
		add(top4);
		
		//Set up the bottom row
		cons.gridx = 1;
	    cons.gridy = 2;
	    cons.insets     = new Insets(0, 0, 0, 0);
		gridBag.setConstraints(bottom1, cons);
		add(bottom1);
		
		cons.gridx = 2;
	    cons.gridy = 2;
	    cons.insets     = new Insets(0, 0, 0, 0);
		gridBag.setConstraints(bottom2, cons);
		add(bottom2);
		
		cons.gridx = 3;
	    cons.gridy = 2;
	    cons.insets     = new Insets(0, 0, 0, 0);
		gridBag.setConstraints(bottom3, cons);
		add(bottom3);
		
		cons.gridx = 4;
	    cons.gridy = 2;
	    cons.insets     = new Insets(0, 0, 0, 0);
		gridBag.setConstraints(bottom4, cons);
		add(bottom4);
	}
	
	/** 
	 *  Gets the question that is on the screen at the moment
	 *  @return KMapType The currently displayed question that is on screen
	 */
	public KMapType getCurrentKMapQuestion() {
		return question;
	}
	
	/** 
	 * generates and displays a new question on the screen
	 */
	public void generateQuestion() {	
		question = generateRandomKMap();
		displayKMap(question);
	}
	
	/** 
	 * Returns a random KMap (not all 1's or zeros).
	 * @return KMapType A randomly generated KMap
	 */
	private KMapType generateRandomKMap() {		
		return new KMapType();//This will generate a random KMap already populated
	}
	
	/** 
	 * Displays the supplied KMap on the screen
	 * @param kMap The kmap to be displayed on screen in the available boxes
	 */
	private void displayKMap(KMapType kMap) {
		
		Vector top    = kMap.getTopRow();
		Vector bottom = kMap.getBottomRow();
		
		//Next put them into the right boxes
		top1.setText(((Integer)top.elementAt(0)).toString());
		top2.setText(((Integer)top.elementAt(1)).toString());
		top3.setText(((Integer)top.elementAt(2)).toString());
		top4.setText(((Integer)top.elementAt(3)).toString());
		
		bottom1.setText(((Integer)bottom.elementAt(0)).toString());
		bottom2.setText(((Integer)bottom.elementAt(1)).toString());
		bottom3.setText(((Integer)bottom.elementAt(2)).toString());
		bottom4.setText(((Integer)bottom.elementAt(3)).toString());
	}
	
	/**
	 * Gets the Default constraints
	 * 
	 * @return   GridBagConstraints The default constraints
	 */
	private GridBagConstraints getDefaultConstraints()
	{
		GridBagConstraints con = new GridBagConstraints();

		con.anchor     = GridBagConstraints.NORTH;
		con.fill       = GridBagConstraints.NONE;
		con.gridheight = 1;
		con.gridwidth  = 1;
		con.gridx      = GridBagConstraints.RELATIVE;
		con.gridy      = GridBagConstraints.RELATIVE;
		con.insets     = new Insets(5, 5, 5, 5);
		con.ipadx      = 0;
		con.ipady      = 0;
		con.weightx    = 0;
		con.weighty    = 0;

		return con;
	}
}